import { checkSchema } from 'express-validator';
import { UserRoles } from '@core';

export const ReadUserSchema = () =>
  checkSchema({
    uuid: {
      isUUID: {
        options: 4,
      },
    },
  });

export const LoginUserSchema = () =>
  checkSchema({
    username: {
      isString: true,
    },
    password: {
      isString: true,
    },
  });

export const LogoutUserSchema = () => checkSchema({});

export const CreateUserSchema = () =>
  checkSchema({
    username: {
      isString: true,
    },
    password: {
      isString: true,
    },
    role: {
      isIn: {
        options: [Object.values(UserRoles)],
      },
    },
  });

export const UpdateUserSchema = () =>
  checkSchema({
    username: {
      isString: true,
      optional: true,
    },
    password: {
      isString: true,
      optional: true,
    },
    role: {
      isIn: {
        options: [Object.values(UserRoles)],
      },
      optional: true,
    },
  });
